c ParaDyn - Parallel DYNAMO - molecular dynamics with EAM potentials
c 
c Authored by Steve Plimpton
c   (505) 845-7873, sjplimp@cs.sandia.gov
c   Dept 9221, MS 1111, Sandia National Labs, Albuquerque, NM  87185-1111
c
c Based on the serial DYNAMO code authored by
c   Stephen Foiles (foiles@ca.sandia.gov), Sandia National Labs, Livermore, CA
c   Murray Daw (daw@hubcap.clemson.edu), Clemson University
c
c See the README file for more information
c
      
      subroutine setup_constraint(inparam1,inparam2,inparam3,inparam4)
#include "paradyn.h"
      
      if (node.eq.0) write (6,*) 'Constraining Atom(s) ...'
      
      if (natoms.eq.0) call error('No atoms defined')
      
      if (inparam1.eq.1) then
        
        do i = 1,nlocal
          if (tag(i).eq.inparam3) fixtag(i) = inparam2
        enddo

      else if (inparam1.eq.2) then
        
        do i = 1,nlocal
          if (tag(i).ge.inparam3.and.tag(i).le.inparam4)
     $         fixtag(i) = inparam2
        enddo

      else if (inparam1.eq.3) then
        
        do i = 1,nlocal
          if (x(1,i).ge.fixreg(1,1,inparam2).and.
     $         x(1,i).le.fixreg(2,1,inparam2).and.
     $         x(2,i).ge.fixreg(1,2,inparam2).and.
     $         x(2,i).le.fixreg(2,2,inparam2).and.
     $         x(3,i).ge.fixreg(1,3,inparam2).and.
     $         x(3,i).le.fixreg(2,3,inparam2)) fixtag(i) = inparam2
        enddo
        
      else if (inparam1.eq.4) then
        
        do i = 1,nlocal
          if (inparam2.eq.0.or.fixtag(i).eq.inparam2) fixtag(i) = 0
        enddo
        
      endif
      
      return
      end
      
      
      
      subroutine fix_correct
#include "paradyn.h"
      
      do i = 1,nlocal
        
        if (fixtag(i).ne.0) then
          
          k = fixtag(i)
          
          if (fixkind(k).eq.1) then
            
            f(1,i) = 0.0D0
            f(2,i) = 0.0D0
            f(3,i) = 0.0D0
            
          else if (fixkind(k).eq.2) then
            
            dot = f(1,i)*fixvec(1,k) +
     $           f(2,i)*fixvec(2,k) + f(3,i)*fixvec(3,k)
            f(1,i) = dot*fixvec(1,k)
            f(2,i) = dot*fixvec(2,k)
            f(3,i) = dot*fixvec(3,k)
            
          else if (fixkind(k).eq.3) then
            
            dot = f(1,i)*fixvec(1,k) +
     $           f(2,i)*fixvec(2,k) + f(3,i)*fixvec(3,k)
            f(1,i) = f(1,i) - dot*fixvec(1,k)
            f(2,i) = f(2,i) - dot*fixvec(2,k)
            f(3,i) = f(3,i) - dot*fixvec(3,k)
            
          else if (fixkind(k).eq.4) then
            
            f(1,i) = f(1,i) + fixvec(1,k)
            f(2,i) = f(2,i) + fixvec(2,k)
            f(3,i) = f(3,i) + fixvec(3,k)
            
c need to put this back in eventually in a new form if want to track work
c watch out for bndtyp flag
c
c            dx = rr(1,i)-posfx(1,i)
c            dy = rr(2,i)-posfx(2,i)
c            dz = rr(3,i)-posfx(3,i)
c            dx = dx - perlen(1)*nint(dx/perlen(1))
c            dy = dy - perlen(2)*nint(dy/perlen(2))
c            dz = dz - perlen(3)*nint(dz/perlen(3))
c            workfx(i) = workfx(i) + vectfx(1,ipntfx(i))*dx +
c     $           vectfx(2,ipntfx(i))*dy + vectfx(3,ipntfx(i))*dz
c            e(i) = e(i) - workfx(i)
c            posfx(1,i) = rr(1,i)
c            posfx(2,i) = rr(2,i)
c            posfx(3,i) = rr(3,i)
            
          endif
          
        endif
        
      enddo
      
      return
      end
