c ParaDyn - Parallel DYNAMO - molecular dynamics with EAM potentials
c 
c Authored by Steve Plimpton
c   (505) 845-7873, sjplimp@cs.sandia.gov
c   Dept 9221, MS 1111, Sandia National Labs, Albuquerque, NM  87185-1111
c
c Based on the serial DYNAMO code authored by
c   Stephen Foiles (foiles@ca.sandia.gov), Sandia National Labs, Livermore, CA
c   Murray Daw (daw@hubcap.clemson.edu), Clemson University
c
c See the README file for more information
c
      
      subroutine setup_control(inparam1,inparam2)
#include "paradyn.h"
      
      if (node.eq.0) write (6,*) 'Controlling Atom(s) ...'
      
      if (natoms.eq.0) call error('No atoms defined')
      
      if (inparam1.eq.1) then
        
        do i = 1,nlocal
          if (x(1,i).ge.tmpreg(1,1,inparam2).and.
     $         x(1,i).le.tmpreg(2,1,inparam2).and.
     $         x(2,i).ge.tmpreg(1,2,inparam2).and.
     $         x(2,i).le.tmpreg(2,2,inparam2).and.
     $         x(3,i).ge.tmpreg(1,3,inparam2).and.
     $         x(3,i).le.tmpreg(2,3,inparam2)) tmptag(i) = inparam2
        enddo
        
      else if (inparam1.eq.2) then
        
        do i = 1,nlocal
          if (inparam2.eq.0.or.tmptag(i).eq.inparam2) tmptag(i) = 0
        enddo
        
      endif
      
      return
      end
      
      
      subroutine tmp_correct
#include "paradyn.h"
      
      call update_controls
      call temperature_controls
      
c set target temperature for each control region
      
      do i = 1,ntemps
        if (tmpstart(i).ge.0.0D0.and.ntimes.ne.0) then
          target(i) = tmpstart(i) + (tmpend(i)-tmpstart(i)) *
     $         (time_stamp-time_mark) / (ntimes*dt)
        else
          target(i) = tmpend(i)
        endif
      enddo
      
c compute drags of each control region
      
      do i = 1,ntemps
        if (tmpkind(i).eq.2) then
          drag(i) = (tmpnow(i)-target(i)) / tmpq(i)
        else if (tmpkind(i).eq.3) then
          dragdot = (tmpnow(i)-target(i)) / tmpq(i)
          drag(i) = drag(i) + dragdot*dt
        endif
      enddo

c correct force on each atom
      
      do i = 1,nlocal
        
        if (tmptag(i).ne.0) then
          
          k = tmptag(i)
          
          if (tmpkind(k).eq.2) then
            
            f(1,i) = f(1,i) - drag(k)*v(1,i)
            f(2,i) = f(2,i) - drag(k)*v(2,i)
            f(3,i) = f(3,i) - drag(k)*v(3,i)
            
          else if (tmpkind(k).eq.3) then
            
            f(1,i) = f(1,i) - drag(k)*v(1,i)
            f(2,i) = f(2,i) - drag(k)*v(2,i)
            f(3,i) = f(3,i) - drag(k)*v(3,i)
            
          else if (tmpkind(k).eq.4) then
            
            stoch = sqrt(2.0D0*tmpq(k)*boltz*target(k)/dt) *
     $           rgauss(tmpseed(i))
            f(1,i) = f(1,i) - tmpq(k)*v(1,i) + stoch
            stoch = sqrt(2.0D0*tmpq(k)*boltz*target(k)/dt) *
     $           rgauss(tmpseed(i))
            f(2,i) = f(2,i) - tmpq(k)*v(2,i) + stoch
            stoch = sqrt(2.0D0*tmpq(k)*boltz*target(k)/dt) *
     $           rgauss(tmpseed(i))
            f(3,i) = f(3,i) - tmpq(k)*v(3,i) + stoch
            
          endif
          
        endif
        
      enddo
      
      return
      end
      
      
      subroutine update_controls
#include "paradyn.h"
      
      do k = 1,ntemps
        if (tmpflag(k).eq.1) then
          do i = 1,nlocal
            if (tmptag(i).eq.k) tmptag(i) = 0
            if (x(1,i).ge.tmpreg(1,1,k).and.
     $           x(1,i).le.tmpreg(2,1,k).and.
     $           x(2,i).ge.tmpreg(1,2,k).and.
     $           x(2,i).le.tmpreg(2,2,k).and.
     $           x(3,i).ge.tmpreg(1,3,k).and.
     $           x(3,i).le.tmpreg(2,3,k)) tmptag(i) = k
          enddo
        endif
      enddo
      
      return
      end
      
      
      subroutine temperature_controls
#include "paradyn.h"
#include "mpif.h"
      
      do i = 0,ntemps
        tmpnum(i) = 0
        tmpnow(i) = 0.0D0
      enddo
      
      do i = 1,nlocal
        k = tmptag(i)
        tmpnum(k) = tmpnum(k) + 1
        tmpnow(k) = tmpnow(k) + amass(itype(i)) *
     $       (v(1,i)*v(1,i) + v(2,i)*v(2,i) + v(3,i)*v(3,i))
      enddo
      
      call mpi_allreduce(tmpnum,tmpiwork,ntemps+1,
     $     mpi_integer,mpi_sum,mpi_comm_world,ierror)
      call mpi_allreduce(tmpnow,tmpwork,ntemps+1,
     $     mpi_double_precision,mpi_sum,mpi_comm_world,ierror)
        
      do i = 0,ntemps
        tmpnum(i) = tmpiwork(i)
        tmpnow(i) = tmpwork(i)
        if (tmpnum(i).gt.0) then
          tmpnow(i) = tmpnow(i) / (3.0D0*boltz*tmpnum(i))
        else
          tmpnow(i) = 0.0D0
        endif
      enddo
      
      return
      end
