c ParaDyn - Parallel DYNAMO - molecular dynamics with EAM potentials
c 
c Authored by Steve Plimpton
c   (505) 845-7873, sjplimp@cs.sandia.gov
c   Dept 9221, MS 1111, Sandia National Labs, Albuquerque, NM  87185-1111
c
c Based on the serial DYNAMO code authored by
c   Stephen Foiles (foiles@ca.sandia.gov), Sandia National Labs, Livermore, CA
c   Murray Daw (daw@hubcap.clemson.edu), Clemson University
c
c See the README file for more information
c
      
      subroutine create_velocities
#include "paradyn.h"
#include "mpif.h"
      
      if (node.eq.0) write (6,*) 'Creating Velocities ...'
      
      if (natoms.eq.0) call error('No atoms defined')
      nvelocities = 1
      
c set velocities so that they are same independent of # of nodes
c  also find velocity of center of mass of free atoms
      
      vxcm = 0.0D0
      vycm = 0.0D0
      vzcm = 0.0D0
      totmass = 0.0D0

      do n = 1,natoms
        tmpx = rgauss(iseed)
        tmpy = rgauss(iseed)
        tmpz = rgauss(iseed)
        if (n.ge.nstart.and.n.le.nend) then
          i = n - nstart + 1
          rmass = amass(itype(i))
          k = tmptag(i)
          if (k.eq.0) then
            vnorm = sqrt(t_request*boltz/rmass)
            v(1,i) = vnorm * tmpx
            v(2,i) = vnorm * tmpy
            v(3,i) = vnorm * tmpz
            totmass = totmass + rmass
            vxcm = vxcm + rmass*v(1,i)
            vycm = vycm + rmass*v(2,i)
            vzcm = vzcm + rmass*v(3,i)
          else if (tmpkind(k).lt.5) then
            tmp = tmpstart(k)
            if (tmp.lt.0.0D0) tmp = t_request
            vnorm = sqrt(tmp*boltz/rmass)
            v(1,i) = vnorm * tmpx
            v(2,i) = vnorm * tmpy
            v(3,i) = vnorm * tmpz
            totmass = totmass + rmass
            vxcm = vxcm + rmass*v(1,i)
            vycm = vycm + rmass*v(2,i)
            vzcm = vzcm + rmass*v(3,i)
          else
            v(1,i) = tmpvec(1,k)
            v(2,i) = tmpvec(2,k)
            v(3,i) = tmpvec(3,k)
          endif
        endif
      enddo
      
      call mpi_allreduce(totmass,tmp,1,mpi_double_precision,
     $     mpi_sum,mpi_comm_world,ierror)
      totmass = tmp
      call mpi_allreduce(vxcm,tmp,1,mpi_double_precision,
     $     mpi_sum,mpi_comm_world,ierror)
      vxcm = tmp
      call mpi_allreduce(vycm,tmp,1,mpi_double_precision,
     $     mpi_sum,mpi_comm_world,ierror)
      vycm = tmp
      call mpi_allreduce(vzcm,tmp,1,mpi_double_precision,
     $     mpi_sum,mpi_comm_world,ierror)
      vzcm = tmp

c subtract center of mass drift for all atoms except those with 
c  specified velocities (tmpkind = 5)
      
      do i = 1,nlocal
        k = tmptag(i)
        if (k.eq.0.or.tmpkind(k).lt.5) then
          v(1,i) = v(1,i) - vxcm/totmass
          v(2,i) = v(2,i) - vycm/totmass
          v(3,i) = v(3,i) - vzcm/totmass
        endif
      enddo

c scale velocities to desired temperature
      
      call scale_velocities

c explicitly alter velocity components of constrained atoms 
c  to satisfy constraints to point, line, plane
c do not change initial velocity if tmpkind = 5 = explicit velocity setting

      do i = 1,nlocal
        if (fixtag(i).ne.0) then
          k = fixtag(i)
          kk = tmptag(i)
          if (kk.eq.0.or.tmpkind(kk).lt.5) then
            if (fixkind(k).eq.1) then
              v(1,i) = 0.0
              v(2,i) = 0.0
              v(3,i) = 0.0
            else if (fixkind(k).eq.2) then
              dot = v(1,i)*fixvec(1,k) +
     $             v(2,i)*fixvec(2,k) + v(3,i)*fixvec(3,k)
              v(1,i) = dot*fixvec(1,k)
              v(2,i) = dot*fixvec(2,k)
              v(3,i) = dot*fixvec(3,k)
            else if (fixkind(k).eq.3) then
              dot = v(1,i)*fixvec(1,k) +
     $             v(2,i)*fixvec(2,k) + v(3,i)*fixvec(3,k)
              v(1,i) = v(1,i) - dot*fixvec(1,k)
              v(2,i) = v(2,i) - dot*fixvec(2,k)
              v(3,i) = v(3,i) - dot*fixvec(3,k)
            endif
          endif
        endif
      enddo
      
c initialize boundary velocities

      bndvel(1) = 0.0D0
      bndvel(2) = 0.0D0
      bndvel(3) = 0.0D0

      return
      end
