c ParaDyn - Parallel DYNAMO - molecular dynamics with EAM potentials
c 
c Authored by Steve Plimpton
c   (505) 845-7873, sjplimp@cs.sandia.gov
c   Dept 9221, MS 1111, Sandia National Labs, Albuquerque, NM  87185-1111
c
c Based on the serial DYNAMO code authored by
c   Stephen Foiles (foiles@ca.sandia.gov), Sandia National Labs, Livermore, CA
c   Murray Daw (daw@hubcap.clemson.edu), Clemson University
c
c See the README file for more information
c
      
      subroutine input(iopflag,inparam1,inparam2,inparam3,inparam4)
#include "paradyn.h"
#include "mpif.h"
      external length
      integer intread
      real*8 realread
      logical match
      character*160 str
      character*80 strtmp,logfile
 900  format (a)
      
      do infinity = 1,1000000000

        if (node.eq.0) read (5,900,end=999,err=999) str
        call mpi_bcast(str,160,mpi_character,0,mpi_comm_world,ierror)

        if (length(str).eq.0) then
          
          continue
          
        else if (match('#',str,m)) then
          
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,900) str(1:length(str))
          
        else if (match('log file',str,m)) then
          
          call strread(str,m,logfile)
          logflag = 1
          if (node.eq.0) then
            close (1)
            open (1,file=logfile)
            close (1,status='delete')
            open (unit=1,file=logfile)
            write (1,*) version(1:length(version))
          endif
          
        else if (match('parallel method',str,m)) then
          
          pmeth = intread(str,m)
          natoms = 0
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Parallel Method',pmeth
          
        else if (match('newton flag',str,m)) then
          
          newton = intread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Newton Flag',newton
          
        else if (match('neighbor method',str,m)) then
          
          nmeth = intread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Neighbor Method',nmeth
          
        else if (match('neighbor cutoff',str,m)) then
          
          dradn = realread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Neighbor Cutoff',dradn
          
        else if (match('boundary method',str,m)) then
          
          ibdtyp = intread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Boundary Method',ibdtyp
          
        else if (match('boundary flags',str,m)) then
          
          idynper(1) = intread(str,m)
          idynper(2) = intread(str,m)
          idynper(3) = intread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Boundary Flags',
     $         idynper(1),idynper(2),idynper(3)
          
        else if (match('desired pressure',str,m)) then
          
          dpress = realread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Desired Pressure',dpress
          
        else if (match('applied pressure',str,m)) then
          
          dstress(1) = realread(str,m)
          dstress(2) = realread(str,m)
          dstress(3) = realread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Applied Pressure',
     $         dstress(1),dstress(2),dstress(3)
          
        else if (match('thermo output',str,m)) then
          
          ithermo = intread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Thermo Output',ithermo
          
        else if (match('atom output',str,m)) then
          
          idump = intread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Atom Output',idump
          
        else if (match('dump file',str,m)) then
          
          call strread(str,m,dumpfile)
          if (node.eq.0) then
            if (logflag.ne.0) write (1,*)
     $           'Dump File ',dumpfile(1:length(dumpfile))
            if (idumpflag.eq.1) close (2)
            open (unit=2,file=dumpfile)
            close (2,status='delete')
            open (unit=2,file=dumpfile)
          endif
          idumpflag = 1

        else if (match('velocity output',str,m)) then
          
          idumpvel = intread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Velocity Output',idumpvel
          
        else if (match('velocity file',str,m)) then
          
          call strread(str,m,dumpvelfile)
          if (node.eq.0) then
            if (logflag.ne.0) write (1,*)
     $           'Velocity File ',dumpvelfile(1:length(dumpvelfile))
            if (idumpvelflag.eq.1) close (3)
            open (unit=3,file=dumpvelfile)
            close (3,status='delete')
            open (unit=3,file=dumpvelfile)
          endif
          idumpvelflag = 1
          
#ifdef STRESS
        else if (match('stress output',str,m)) then
          
          idumpstrs = intread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Stress Output',idumpstrs
          
        else if (match('stress file',str,m)) then
          
          call strread(str,m,dumpstrsfile)
          if (node.eq.0) then
            if (logflag.ne.0) write (1,*)
     $           'Stress File ',dumpstrsfile(1:length(dumpstrsfile))
            if (idumpstrsflag.eq.1) close (3)
            open (unit=4,file=dumpstrsfile)
            close (4,status='delete')
            open (unit=4,file=dumpstrsfile)
          endif
          idumpstrsflag = 1

#endif
        else if (match('constraint style',str,m)) then

          itmp = intread(str,m)
          if (itmp.eq.0) then
            nfixes = 0
            if (logflag.ne.0.and.node.eq.0)
     $           write (1,*) 'Constraint Style',itmp
          else
            nfixes = max(nfixes,itmp)
            if (nfixes.gt.maxfix)
     $           call error('Too many constraints - boost maxfix')
            fixkind(itmp) = intread(str,m)
            fixvec(1,itmp) = 0.0D0
            fixvec(2,itmp) = 0.0D0
            fixvec(3,itmp) = 0.0D0
            if (logflag.ne.0.and.node.eq.0)
     $           write (1,*) 'Constraint Style',
     $           itmp,fixkind(itmp)
          endif
          
        else if (match('constraint vector',str,m)) then
          
          itmp = intread(str,m)
          if (itmp.gt.nfixes) call error('Bad constraint #')
          if (fixkind(itmp).eq.1) call error('No vector allowed')
          fixvec(1,itmp) = realread(str,m)
          fixvec(2,itmp) = realread(str,m)
          fixvec(3,itmp) = realread(str,m)
          if (fixkind(itmp).ne.4) then
            sum = fixvec(1,itmp)*fixvec(1,itmp) +
     $           fixvec(2,itmp)*fixvec(2,itmp) +
     $           fixvec(3,itmp)*fixvec(3,itmp)
            sum = sqrt(sum)
            if (sum.ne.0.0D0) then
              fixvec(1,itmp) = fixvec(1,itmp) / sum
              fixvec(2,itmp) = fixvec(2,itmp) / sum
              fixvec(3,itmp) = fixvec(3,itmp) / sum
            endif
          endif
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Constraint Vector',itmp,
     $         (fixvec(i,itmp),i=1,3)
          
        else if (match('temperature style',str,m)) then
          
          itmp = intread(str,m)
          if (itmp.eq.0) then
            ntemps = 0
            if (logflag.ne.0.and.node.eq.0)
     $           write (1,*) 'Temperature Style',itmp
          else
            ntemps = max(ntemps,itmp)
            if (ntemps.gt.maxtmp)
     $           call error('Too many controls - boost maxtmp')
            tmpkind(itmp) = intread(str,m)
            if (tmpkind(itmp).eq.4) langseed = intread(str,m)
            tmpstart(itmp) = -1.0D20
            tmpend(itmp) = 0.0D0
            tmpflag(itmp) = 0
            if (tmpkind(itmp).eq.2) then
              tmpq(itmp) = 1730.0D0
            else if (tmpkind(itmp).eq.3) then
              tmpq(itmp) = 40.0D0
            else if (tmpkind(itmp).eq.4) then
              tmpq(itmp) = 0.1D0
            endif
            tmpvec(1,itmp) = 0.0D0
            tmpvec(2,itmp) = 0.0D0
            tmpvec(3,itmp) = 0.0D0
            if (tmpkind(itmp).ne.4) then
              if (logflag.eq.0.and.node.eq.0)
     $             write (1,*) 'Temperature Style',
     $             itmp,tmpkind(itmp)
            else
              if (logflag.eq.0.and.node.eq.0)
     $             write (1,*) 'Temperature Style',
     $             itmp,tmpkind(itmp),langseed
            endif
          endif
          
        else if (match('temperature flags',str,m)) then
          
          itmp = intread(str,m)
          if (itmp.gt.ntemps) call error('Bad control #')
          mm = m
          call strread(str,m,strtmp)
          if (strtmp(1:3).eq.'---') then
            tmpstart(itmp) = -1.0D20
          else
            tmpstart(itmp) = realread(str,mm)
            m = mm
          endif
          tmpend(itmp) = realread(str,m)
          tmpflag(itmp) = intread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Temperature Flags',
     $         itmp,tmpstart(itmp),tmpend(itmp),tmpflag(itmp)
          
        else if (match('temperature vector',str,m)) then
          
          itmp = intread(str,m)
          if (itmp.gt.ntemps) call error('Bad control #')
          if (tmpkind(itmp).eq.1) call error('No vector allowed')
          tmpvec(1,itmp) = realread(str,m)
          tmpvec(2,itmp) = realread(str,m)
          tmpvec(3,itmp) = realread(str,m)
          
        else if (match('temperature constant',str,m)) then
          
          itmp = intread(str,m)
          if (itmp.gt.ntemps) call error('Bad control #')
          tmpq(itmp) = realread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Temperature Constant',tmpq(itmp)
          
        else if (match('diagnostic',str,m)) then
          
          idiag = intread(str,m)
          if (idiag.eq.0) then
            if (logflag.ne.0.and.node.eq.0)
     $           write (1,*) 'Diagnostic',idiag
          else
            call strread(str,m,diagfile)
            ndiag = intread(str,m)
            diag1 = 0.0D0
            diag2 = 0.0D0
            diag3 = 0.0D0
            diag4 = 0.0D0
            diag5 = 0.0D0
            if (ndiag.ge.1) diag1 = realread(str,m)
            if (ndiag.ge.2) diag2 = realread(str,m)
            if (ndiag.ge.3) diag3 = realread(str,m)
            if (ndiag.ge.4) diag4 = realread(str,m)
            if (ndiag.ge.5) diag5 = realread(str,m)
            if (logflag.ne.0.and.node.eq.0)
     $           write (1,*) 'Diagnostic',idiag,' ',
     $           diagfile(1:length(diagfile)),ndiag,
     $           diag1,diag2,diag3,diag4,diag5
          endif

        else if (match('user force',str,m)) then
          
          iuser = intread(str,m)
          if (iuser.eq.0) then
            if (logflag.ne.0.and.node.eq.0)
     $           write (1,*) 'User force',iuser
          else
            user1 = 0.0D0
            user2 = 0.0D0
            user3 = 0.0D0
            user4 = 0.0D0
            user5 = 0.0D0
            if (iuser.ge.1) user1 = realread(str,m)
            if (iuser.ge.2) user2 = realread(str,m)
            if (iuser.ge.3) user3 = realread(str,m)
            if (iuser.ge.4) user4 = realread(str,m)
            if (iuser.ge.5) user5 = realread(str,m)
            if (logflag.ne.0.and.node.eq.0)
     $           write (1,*) 'User force',iuser,
     $           user1,user2,user3,user4,user5
          endif
          
        else if (match('read potential',str,m)) then
          
          iopflag = 1
          ipot = intread(str,m)
          call strread(str,m,potfile)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Read Potential',ipot,
     $         ' ',potfile(1:length(potfile))
          return
          
        else if (match('interpolate',str,m)) then
          
          iopflag = 2
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Interpolate Potentials'
          return
          
        else if (match('create atoms',str,m)) then
          
          iopflag = 4
          call strread(str,m,lattype)
          latconst = realread(str,m)
          xlat = intread(str,m)
          ylat = intread(str,m)
          zlat = intread(str,m)
          ilatseed = intread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Create Atoms ',lattype(1:3),
     $         latconst,xlat,ylat,zlat,ilatseed
          return
          
        else if (match('read atoms',str,m)) then
          
          iopflag = 3
          call strread(str,m,atomfile)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Read Atoms ',atomfile(1:length(atomfile))
          return
          
        else if (match('constrain atom',str,m)) then
          
          iopflag = 5
          inparam1 = 1
          inparam2 = intread(str,m)
          if (inparam2.gt.nfixes) call error('Bad constraint #')
          inparam3 = intread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Constrain Atom',inparam2,inparam3
          return
          
        else if (match('constrain group',str,m)) then
          
          iopflag = 5
          inparam1 = 2
          inparam2 = intread(str,m)
          if (inparam2.gt.nfixes) call error('Bad constraint #')
          inparam3 = intread(str,m)
          inparam4 = intread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Constrain Group',
     $         inparam2,inparam3,inparam4
          return
          
        else if (match('constrain region',str,m)) then
          
          iopflag = 5
          inparam1 = 3
          inparam2 = intread(str,m)
          if (inparam2.gt.nfixes) call error('Bad constraint #')
          do i = 1,3
            mm = m
            call strread(str,m,strtmp)
            if (strtmp(1:3).eq.'---') then
              fixreg(1,i,inparam2) = -1.0D20
            else
              fixreg(1,i,inparam2) = realread(str,mm)
              m = mm
            endif
            mm = m
            call strread(str,m,strtmp)
            if (strtmp(1:3).eq.'---') then
              fixreg(2,i,inparam2) = 1.0D20
            else
              fixreg(2,i,inparam2) = realread(str,mm)
              m = mm
            endif
          enddo
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Constrain Region',inparam2,
     $         ((fixreg(i,j,inparam2),i=1,2),j=1,3)
          return
          
        else if (match('constrain off',str,m)) then
          
          iopflag = 5
          inparam1 = 4
          inparam2 = intread(str,m)
          if (inparam2.gt.nfixes) call error('Bad constraint #')
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Constrain Off',inparam2
          return
          
        else if (match('control region',str,m)) then
          
          iopflag = 6
          inparam1 = 1
          inparam2 = intread(str,m)
          if (inparam2.gt.ntemps) call error('Bad control #')
          do i = 1,3
            mm = m
            call strread(str,m,strtmp)
            if (strtmp(1:3).eq.'---') then
              tmpreg(1,i,inparam2) = -1.0D20
            else
              tmpreg(1,i,inparam2) = realread(str,mm)
              m = mm
            endif
            mm = m
            call strread(str,m,strtmp)
            if (strtmp(1:3).eq.'---') then
              tmpreg(2,i,inparam2) = 1.0D20
            else
              tmpreg(2,i,inparam2) = realread(str,mm)
              m = mm
            endif
          enddo
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Control Region',inparam2,
     $         ((tmpreg(i,j,inparam2),i=1,2),j=1,3)
          return
          
        else if (match('control off',str,m)) then
          
          iopflag = 6
          inparam1 = 2
          inparam2 = intread(str,m)
          if (inparam2.gt.ntemps) call error('Bad control #')
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Control Off',inparam2
          return
          
        else if (match('create velocities',str,m)) then
          
          iopflag = 8
          t_request = realread(str,m)
          iseed = intread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Create Velocities',t_request,iseed
          return
          
        else if (match('read velocities',str,m)) then
          
          iopflag = 7
          call strread(str,m,velfile)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Read Velocities ',velfile(1:length(velfile))
          return
          
        else if (match('minimize',str,m)) then
          
          iopflag = 9
          maxstep = intread(str,m)
          tolerance = realread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Minimize',maxstep,tolerance
          return
          
        else if (match('run',str,m)) then
          
          iopflag = 10
          ntimes = intread(str,m)
          dt = realread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Run',ntimes,dt
          return
          
        else if (match('scale atoms',str,m)) then
          
          iopflag = 11
          xscale = realread(str,m)
          yscale = realread(str,m)
          zscale = realread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Scale Atoms',xscale,yscale,zscale
          return

        else if (match('scale velocities',str,m)) then
          
          iopflag = 12
          t_request = realread(str,m)
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Scale Velocities',t_request
          return
          
        else if (match('dump atoms',str,m)) then
          
          iopflag = 13
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Dump Atoms'
          return

        else if (match('dump velocities',str,m)) then
          
          iopflag = 14
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Dump Velocities'
          return
          
#ifdef STRESS
        else if (match('dump stresses',str,m)) then
          
          iopflag = 15
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Dump Stresses'
          return
          
#endif
        else if (match('clear',str,m)) then
          
          iopflag = 16
          if (logflag.ne.0.and.node.eq.0)
     $         write (1,*) 'Clear'
          return
          
        else
          
          call error('STRING: '//str(1:length(str)))
          
        endif
        
      enddo
      
 999  continue
      
      str = 'All Done'
      call mpi_bcast(str,160,mpi_character,0,mpi_comm_world,ierror)
      call error('All Done')
      
      return
      end
