c ParaDyn - Parallel DYNAMO - molecular dynamics with EAM potentials
c 
c Authored by Steve Plimpton
c   (505) 845-7873, sjplimp@cs.sandia.gov
c   Dept 9221, MS 1111, Sandia National Labs, Albuquerque, NM  87185-1111
c
c Based on the serial DYNAMO code authored by
c   Stephen Foiles (foiles@ca.sandia.gov), Sandia National Labs, Livermore, CA
c   Murray Daw (daw@hubcap.clemson.edu), Clemson University
c
c See the README file for more information
c
      
      subroutine integrate
#include "paradyn.h"
#include "mpif.h"
      
      call mpi_barrier(mpi_comm_world,ierror)
      time_runstart = mpi_wtime()
      
      time_mark = time_stamp
      
      dt2 = 0.5D0*dt**2
      dtrec = 1.0D0/dt
      
      ndegfr = nlocal
      if (ibdtyp.eq.2) ndegfr = nlocal + 1
      
      call scale
      
      do j = 1,3
        do i = 1,ndegfr
          u3(j,i) = 0.0D0
          u4(j,i) = 0.0D0
          u5(j,i) = 0.0D0
        enddo
      enddo
      
      call accel
      
      do j = 1,3
        do i = 1,ndegfr
          u1(j,i) = dt*y(j+3,i)
          u2(j,i) = dt2*acc(j,i)
        enddo
      enddo
      
      do k = 1,ntimes
        
        time_stamp = time_mark + k*dt
        
        do j = 1,3
          do i = 1,ndegfr
            y(j,i) = y(j,i) + u1(j,i) + u2(j,i) + u3(j,i) +
     $           u4(j,i) + u5(j,i)
            u1(j,i) = u1(j,i) + 2.0D0*u2(j,i) + 3.0D0*u3(j,i) +
     $           4.0D0*u4(j,i) + 5.0D0*u5(j,i)
            y(j+3,i) = dtrec*u1(j,i)
            u2(j,i) = u2(j,i) + 3.0D0*u3(j,i) + 6.0D0*u4(j,i) +
     $           10.0D0*u5(j,i)
            u3(j,i) = u3(j,i) + 4.0D0*u4(j,i) + 10.0D0*u5(j,i)
            u4(j,i) = u4(j,i) + 5.0D0*u5(j,i)
          enddo
        enddo
        
        call accel

        do j = 1,3
          do i = 1,ndegfr
            phi = dt2*acc(j,i) - u2(j,i)
            y(j,i) = y(j,i) + c0*phi
            u1(j,i) = u1(j,i) + c1*phi
            y(j+3,i) = dtrec*u1(j,i)
            u2(j,i) = u2(j,i) + c2*phi
            u3(j,i) = u3(j,i) + c3*phi
            u4(j,i) = u4(j,i) + c4*phi
            u5(j,i) = u5(j,i) + c5*phi
          enddo
        enddo
        
        if (ithermo.ne.0) then
          if (mod(k,ithermo).eq.0) then
            call remap
            call unscale
            call output
          endif
        endif
        
        if (idump.ne.0) then
          if (mod(k,idump).eq.0) then
            call remap
            call unscale
            call dump_atoms
          endif
        endif

        if (idumpvel.ne.0) then
          if (mod(k,idumpvel).eq.0) then
            call unscale
            call dump_velocities
          endif
        endif
        
#ifdef STRESS
        if (idumpstrs.ne.0) then
          if (mod(k,idumpstrs).eq.0) then
            call unscale
            call dump_stresses
          endif
        endif
#endif
        
        if (idiag.ne.0) then
          if (mod(k,idiag).eq.0) then
            call remap
            call unscale
            call diagnostic(k)
          endif
        endif
        
      enddo
      
      call remap
      call unscale
      
      time_stamp = time_mark + ntimes*dt
      
      call mpi_barrier(mpi_comm_world,ierror)
      time_runend = mpi_wtime()
      
      return
      end
      
      
      subroutine accel
#include "paradyn.h"
      
      call remap
      call unscale
      if (ibdtyp.ne.1) call chkper
      call force(1,1,0)
      
      if (ibdtyp.eq.1) then
        
        do i = 1,nlocal
          acc(1,i) = f(1,i)/amass(itype(i))
          acc(2,i) = f(2,i)/amass(itype(i))
          acc(3,i) = f(3,i)/amass(itype(i))
        enddo

      else
        
        do i = 1,nlocal
          do k = 1,3
            acc(k,i) = (f(k,i)/amass(itype(i))
     $           - 2.0D0*y(k+3,ndegfr)*y(k+3,i))/y(k,ndegfr)
          enddo
        enddo
        
        volume = y(1,ndegfr)*y(2,ndegfr)*y(3,ndegfr)
        do k = 1,3
          acc(k,ndegfr) = idynper(k)*(
     $         ((stress(k)-dpress*volume)/y(k,ndegfr)
     $         - y(k,ndegfr)*dstress(k))/bndmas(k)
     $         - bnddrg(k)*y(3+k,ndegfr))
        enddo
        
      endif
      
      return
      end
