c ParaDyn - Parallel DYNAMO - molecular dynamics with EAM potentials
c 
c Authored by Steve Plimpton
c   (505) 845-7873, sjplimp@cs.sandia.gov
c   Dept 9221, MS 1111, Sandia National Labs, Albuquerque, NM  87185-1111
c
c Based on the serial DYNAMO code authored by
c   Stephen Foiles (foiles@ca.sandia.gov), Sandia National Labs, Livermore, CA
c   Murray Daw (daw@hubcap.clemson.edu), Clemson University
c
c See the README file for more information
c
      
      subroutine read_potential
#include "paradyn.h"
#include "mpif.h"
      
 900  format(i5,2g15.5)
 901  format(i5,e24.16,i5,2e24.16)
 902  format(5e24.16)
 903  format(i5)
      
      if (node.eq.0) write (6,*) 'Reading Potential ...'
      
      ninterp = 0
      natoms = 0
      nvelocities = 0
      
c open potential file - exit if there is a problem

      if (node.eq.0) then
        iflag = -1
        open (unit=9,file=potfile,form='formatted',
     $       status='old',err=10)
        iflag = 0
      endif
      
 10   call mpi_bcast(iflag,1,mpi_integer,0,mpi_comm_world,ierror)
      if (iflag.lt.0) call error('Potential file cannot be opened')
      
c read potential data

      if (ipot.gt.0) then
        
        if (ipot.gt.npots+1) call error('Mis-ordered potential')
        npots = max(npots,ipot)
        if (npots.gt.maxtype)
     $       call error('Too many potentials - boost maxtype')
        
        if (node.eq.0) then
          read (9,*)
          read (9,900) ielement(ipot),amass(ipot),alat(ipot)
          read (9,901) nrhoin(ipot),drhoin(ipot),
     $         nrin(ipot),drin(ipot),rcut(ipot)
        endif
        
        call mpi_bcast(ielement(ipot),1,mpi_integer,0,
     $       mpi_comm_world,ierror)
        call mpi_bcast(amass(ipot),1,mpi_double_precision,0,
     $       mpi_comm_world,ierror)
        call mpi_bcast(alat(ipot),1,mpi_double_precision,0,
     $       mpi_comm_world,ierror)

        call mpi_bcast(nrhoin(ipot),1,mpi_integer,0,
     $       mpi_comm_world,ierror)
        call mpi_bcast(drhoin(ipot),1,mpi_double_precision,0,
     $       mpi_comm_world,ierror)
        call mpi_bcast(nrin(ipot),1,mpi_integer,0,
     $       mpi_comm_world,ierror)
        call mpi_bcast(drin(ipot),1,mpi_double_precision,0,
     $       mpi_comm_world,ierror)
        call mpi_bcast(rcut(ipot),1,mpi_double_precision,0,
     $       mpi_comm_world,ierror)
        
        if (nrhoin(ipot).gt.maxgridin.or.nrin(ipot).gt.maxgridin)
     $       call error('Too many grid points - boost maxgridin')
          
        if (node.eq.0) then
          read (9,902) (frhoin(j,ipot),j=1,nrhoin(ipot))
          read (9,902) (zrin(j,ipot),j=1,nrin(ipot))
          read (9,902) (rhorin(j,ipot),j=1,nrin(ipot))
          close (9)
        endif
        
        call mpi_bcast(frhoin(1,ipot),nrhoin(ipot),
     $       mpi_double_precision,0,mpi_comm_world,ierror)
        call mpi_bcast(zrin(1,ipot),nrin(ipot),
     $       mpi_double_precision,0,mpi_comm_world,ierror)
        call mpi_bcast(rhorin(1,ipot),nrin(ipot),
     $       mpi_double_precision,0,mpi_comm_world,ierror)
        
      else
        
        if (node.eq.0) then
          read (9,*) 
          read (9,*)
          read (9,*)
          read (9,903) npots
        endif
          
        call mpi_bcast(npots,1,mpi_integer,0,
     $       mpi_comm_world,ierror)

        if (npots.gt.maxtype)
     $       call error('Too many potentials - boost maxtype')
        
        if (node.eq.0) read (9,901) nrho,drho,nr,dr,rcutmax

        call mpi_bcast(nrho,1,mpi_integer,0,
     $       mpi_comm_world,ierror)
        call mpi_bcast(drho,1,mpi_double_precision,0,
     $       mpi_comm_world,ierror)
        call mpi_bcast(nr,1,mpi_integer,0,
     $       mpi_comm_world,ierror)
        call mpi_bcast(dr,1,mpi_double_precision,0,
     $       mpi_comm_world,ierror)
        call mpi_bcast(rcutmax,1,mpi_double_precision,0,
     $       mpi_comm_world,ierror)
        
        if (nrho.gt.maxgridin.or.nr.gt.maxgridin)
     $       call error('Too many grid points - boost maxgridin')
        
        if (node.eq.0) then
          do i = 1,npots
            read (9,900) ielement(i),amass(i),alat(i)
            read (9,902) (frho(j,i),j=1,nrho)
            read (9,902) (rhor(j,i),j=1,nr)
          enddo
          do i1 = 1,npots
            do i2 = 1,i1
              read (9,902) (z2r(j,i1,i2),j=1,nr)
            enddo
          enddo
          close (9)
        endif
        
        call mpi_bcast(ielement,npots,
     $       mpi_integer,0,mpi_comm_world,ierror)
        call mpi_bcast(amass,npots,
     $       mpi_double_precision,0,mpi_comm_world,ierror)
        call mpi_bcast(alat,npots,
     $       mpi_double_precision,0,mpi_comm_world,ierror)

        call mpi_bcast(frho,maxgridin*npots,
     $       mpi_double_precision,0,mpi_comm_world,ierror)
        call mpi_bcast(rhor,maxgridin*npots,
     $       mpi_double_precision,0,mpi_comm_world,ierror)
        call mpi_bcast(z2r,maxgridin*maxtype*npots,
     $       mpi_double_precision,0,mpi_comm_world,ierror)
      
      endif
      
      return
      end
